@@ -25,7 +25,8 @@ class Pool:
     @classmethod
     @asyncio.coroutine
     def create(cls, host='localhost', port=6379, password=None, db=0,
-               encoder=None, poolsize=1, auto_reconnect=True, loop=None):
+               encoder=None, poolsize=1, auto_reconnect=True, loop=None,
+               protocol_class=RedisProtocol):
         """
         Create a new connection pool instance.
 
@@ -44,6 +45,8 @@ class Pool:
         :param auto_reconnect: Enable auto reconnect
         :type auto_reconnect: bool
         :param loop: (optional) asyncio event loop.
+        :type protocol_class: RedisProtocol
+        :param protocol_class: (optional) redis protocol implementation
         """
         self = cls()
         self._host = host
@@ -56,7 +59,8 @@ class Pool:
         for i in range(poolsize):
             connection = yield from Connection.create(host=host, port=port,
                             password=password, db=db, encoder=encoder,
-                            auto_reconnect=auto_reconnect, loop=loop)
+                            auto_reconnect=auto_reconnect, loop=loop,
+                            protocol_class=protocol_class)
             self._connections.append(connection)
 
         return self
