@@ -7,6 +7,11 @@ from asyncio.futures import Future
 from asyncio.queues import Queue
 from asyncio.streams import StreamReader
 
+try:
+    import hiredis
+except ImportError:
+    hiredis = None
+
 from collections import deque
 from functools import wraps
 from inspect import getfullargspec, formatargspec, getcallargs
@@ -42,6 +47,7 @@ from .cursors import Cursor, SetCursor, DictCursor, ZCursor
 
 __all__ = (
     'RedisProtocol',
+    'HiRedisProtocol',
     'Transaction',
     'Subscription',
     'Script',
@@ -105,44 +111,86 @@ class MultiBulkReply:
     """
     def __init__(self, protocol, count, loop=None):
         self._loop = loop or asyncio.get_event_loop()
-        self.queue = Queue(loop=self._loop)
+
+        #: Buffer of incoming, undelivered data, received from the parser.
+        self._data_queue = []
+
+        #: Incoming read queries.
+        #: Contains (read_count, Future, decode_flag, one_only_flag) tuples.
+        self._f_queue = deque()
+
         self.protocol = protocol
         self.count = int(count)
 
+    def _feed_received(self, item):
+        """
+        Feed entry for the parser.
+        """
+        # Push received items on the queue
+        self._data_queue.append(item)
+        self._flush()
+
+    def _flush(self):
+        """
+        Answer read queries when we have enough data in our multibulk reply.
+        """
+        # As long as we have more data in our queue then we require for a read
+        # query -> answer queries.
+        while self._f_queue and self._f_queue[0][0] <= len(self._data_queue):
+            # Pop query.
+            count, f, decode, one_only = self._f_queue.popleft()
+
+            # Slice data buffer.
+            data, self._data_queue = self._data_queue[:count], self._data_queue[count:]
+
+            # When the decode flag is given, decode bytes to native types.
+            if decode:
+                data = [ self._decode(d) for d in data ]
+
+            # When one_only flag has been given, don't return an array.
+            if one_only:
+                assert len(data) == 1
+                f.set_result(data[0])
+            else:
+                f.set_result(data)
+
+    def _decode(self, result):
+        """ Decode bytes to native Python types. """
+        if isinstance(result, (StatusReply, int, float, MultiBulkReply)):
+            # Note that MultiBulkReplies can be nested. e.g. in the 'scan' operation.
+            return result
+        elif isinstance(result, bytes):
+            return self.protocol.decode_to_native(result)
+        elif result is None:
+            return result
+        else:
+            raise AssertionError('Invalid type: %r' % type(result))
+
+    def _read(self, decode=True, count=1, _one=False):
+        """ Do read operation on the queue. Return future. """
+        f = Future(loop=self.protocol._loop)
+        self._f_queue.append((count, f, decode, _one))
+
+        # If there is enough data on the queue, answer future immediately.
+        self._flush()
+
+        return f
+
     def iter_raw(self):
         """
         Iterate over all multi bulk packets. This yields futures that won't
         decode bytes yet.
         """
         for i in range(self.count):
-            yield self.queue.get()
+            yield self._read(decode=False, _one=True)
 
     def __iter__(self):
         """
         Iterate over the reply. This yields coroutines of the decoded packets.
         It decodes bytes automatically using protocol.decode_to_native.
         """
-        @asyncio.coroutine
-        def auto_decode(f):
-            result = yield from f
-            if isinstance(result, (StatusReply, int, float, MultiBulkReply)):
-                # Note that MultiBulkReplies can be nested. e.g. in the 'scan' operation.
-                return result
-            elif isinstance(result, bytes):
-                return self.protocol.decode_to_native(result)
-            elif result is None:
-                return result
-            else:
-                raise AssertionError('Invalid type: %r' % type(result))
-
-        for f in self.iter_raw():
-            # We should immediately wrap this coroutine in async(), to be sure
-            # that the order of the queue remains, even if we wrap it in
-            # gather:
-            #    f1 = next(multibulk)
-            #    f2 = next(multibulk)
-            #    r1, r2 = gather(f1, f2)
-            yield asyncio.async(auto_decode(f), loop=self._loop)
+        for i in range(self.count):
+            yield self._read(_one=True)
 
     def __repr__(self):
         return 'MultiBulkReply(protocol=%r, count=%r)' % (self.protocol, self.count)
@@ -923,7 +971,7 @@ class RedisProtocol(asyncio.Protocol, metaclass=_RedisProtocolMeta):
 
         # Wait for all multi bulk reply content.
         for i in range(count):
-            yield from self._handle_item(reply.queue.put_nowait)
+            yield from self._handle_item(reply._feed_received)
 
     @asyncio.coroutine
     def _handle_pubsub_multibulk_reply(self, multibulk_reply):
@@ -2290,3 +2338,110 @@ class Subscription:
         :returns: instance of :class:`PubSubReply <asyncio_redis.replies.PubSubReply>`
         """
         return (yield from self._messages_queue.get())
+
+
+class HiRedisProtocol(RedisProtocol, metaclass=_RedisProtocolMeta):
+    def __init__(self, password=None, db=0, encoder=None,
+                 connection_lost_callback=None, enable_typechecking=True,
+                 loop=None):
+        super().__init__(password, db, encoder, connection_lost_callback,
+                         enable_typechecking, loop)
+        self._hiredis = None
+        assert hiredis, "`hiredis` libary not available. Please don't use HiRedisProtocol."
+
+    def connection_made(self, transport):
+        super().connection_made(transport)
+        self._hiredis = hiredis.Reader()
+        self._line_buffer = b''
+        self._reply_type = None
+
+    @asyncio.coroutine
+    def _read_lines(self):
+        """
+        Reads blocks from transport and returns all completed lines.
+
+        Unused data already read from stream is cached for next line.
+        """
+        data = yield from self._reader.read(self._reader._limit)
+        lines = data.splitlines(keepends=True)
+
+        # stick buffered start of line to first received line.
+        if self._line_buffer:
+            lines[0] = self._line_buffer + lines[0]
+
+        # buffer last incomplete line and don't return it
+        if not lines[-1].endswith(b'\r\n'):
+            self._line_buffer = lines[-1]
+            return lines[:-1]
+        self._line_buffer = b''
+        return lines
+
+    @asyncio.coroutine
+    def _handle_item(self, cb):
+        """
+        Feeds HiRedis parser line-by-line, and handle all parsed results.
+
+        if hiredis returns completed reply, passes it to corresponding callback
+        found by first character of first processed line.
+        """
+        lines = yield from self._read_lines()
+        while lines:
+            for line in lines:
+                if self._reply_type is None:
+                    self._reply_type = line[:1]
+                self._hiredis.feed(line)
+                parsed = self._hiredis.gets()
+                if parsed is False:
+                    continue
+                callback = self._line_received_handlers[self._reply_type]
+                yield from callback(cb, parsed)
+                self._reply_type = None
+            lines = yield from self._read_lines()
+
+    @asyncio.coroutine
+    def _handle_status_reply(self, cb, line):
+        cb(StatusReply(line.decode('ascii')))
+
+    @asyncio.coroutine
+    def _handle_int_reply(self, cb, line):
+        cb(line)
+
+    @asyncio.coroutine
+    def _handle_error_reply(self, cb, line):
+        if isinstance(line, hiredis.ReplyError):
+            cb(ErrorReply(line.args[0]))
+            return
+        cb(ErrorReply(line.decode('ascii')))
+
+    @asyncio.coroutine
+    def _handle_bulk_reply(self, cb, result):
+        cb(result)
+
+    @asyncio.coroutine
+    def _handle_multi_bulk_reply(self, cb, result):
+        if result is None:
+            cb(None)
+            return
+
+        reply = self._parse_multi_bulk_from_list(result)
+
+        # Return the empty queue immediately as an answer.
+        if self._in_pubsub:
+            asyncio.async(self._handle_pubsub_multibulk_reply(reply),
+                          loop=self._loop)
+        else:
+            cb(reply)
+
+    def _parse_multi_bulk_from_list(self, data):
+        """
+        Resursively constructs MultyBulkReply from list structure
+        returned by hiredis parser.
+
+        Helpful for parsing transaction and script replies.
+        """
+        reply = MultiBulkReply(self, len(data), loop=self._loop)
+        for token in data:
+            if isinstance(token, list):
+                token = self._parse_multi_bulk_from_list(token)
+            reply._feed_received(token)
+        return reply
